! ############################################################
!            Satellite Data Simulation Unit v2
!                      -  rd_CRM -
!
! This subroutine reads cloud-resolving model profiles.
!
!                   Updated from rd_CRM.f by H. M. (May, 2009)
!
 Subroutine rd_CRM ( ifile )
   Use MAINparameters
   Implicit NONE
!
! I/O parameters
!
   Integer, intent(in) :: ifile
!
! Local varaibles
!
   Integer :: i,j,k,irec
   Real    :: wind_lev
   character :: input_file*90
   character :: pathandfile*90
!      
! Define the input file name
!
   input_file = CRM_file_list(ifile)
   pathandfile = CRMdir//trim(input_file)
   open(unit = 14, file = pathandfile, access = 'DIRECT' , recl = 44 )
!
   print *, 'Input: '//trim(input_file)
!
   irec = 1
!
! Read CRM grid size
!  ngridx and ngridy: #s of grid points along x and y directions.
!  nlyr: # of vertical layers.
!  deltax and deltay: grid intervals along x and y directions.
!   (deltax and deltay will be used only for beam convolution)
!
   Read(14,rec=1) ngridx,ngridy,nlyr,deltax,deltay
!
! Allocate 2D variables
!
   allocate ( ics(ngridx,ngridy), tskin(ngridx,ngridy) )
   allocate ( rainrate_sfc(ngridx,ngridy), sfc_type(ngridx,ngridy) )
   allocate ( wind_sfc(ngridx,ngridy), soilH2O(ngridx,ngridy) )
   allocate ( albd_visir(ngridx,ngridy,mxwavel) )
!
! Allocate 3D variables defined at layer interfaces
!
   allocate (  hght_lev(ngridx,ngridy,0:nlyr) ) 
   allocate (  pres_lev(ngridx,ngridy,0:nlyr) ) 
   allocate (  temp_lev(ngridx,ngridy,0:nlyr) ) 
!
! Allocate 3D variables defined as layer averages
!
   allocate (  rlhm_avg(ngridx,ngridy,nlyr)   ) 
   allocate ( Wcldw_avg(ngridx,ngridy,nlyr)   ) 
   allocate ( Wrain_avg(ngridx,ngridy,nlyr)   ) 
   allocate ( Wcldi_avg(ngridx,ngridy,nlyr)   ) 
   allocate ( Wsnow_avg(ngridx,ngridy,nlyr)   ) 
   allocate ( Wgrpl_avg(ngridx,ngridy,nlyr)   ) 
   allocate ( Whail_avg(ngridx,ngridy,nlyr)   ) 
   allocate ( Ncldw_avg(ngridx,ngridy,nlyr)   ) 
   allocate ( Nrain_avg(ngridx,ngridy,nlyr)   ) 
   allocate ( Ncldi_avg(ngridx,ngridy,nlyr)   ) 
   allocate ( Nsnow_avg(ngridx,ngridy,nlyr)   ) 
   allocate ( Ngrpl_avg(ngridx,ngridy,nlyr)   ) 
   allocate ( Nhail_avg(ngridx,ngridy,nlyr)   ) 
!
! --------------- Read CRM profiles -----------------
!
! surface height is fixed at 0 km.
!
   hght_lev(:,:,0) = 0.
!
   do i = 1, ngridx
      do j = 1, ngridy
          irec = irec + 1
          Read(14,rec=irec) &
!
!             ### READ IN SURFACE PARAMETERS ###
! surface pressure in [hPa] and surface air temperature in [K]
!
          pres_lev(i,j,0), temp_lev (i,j,0), &
!
! surface skin temperature in [K] and convective/stratiform index
! (ics=1 for convective and ics=2 for stratiform. The dielectric
!  parameterization of melting particles are applied to profiles w ics=2.)
! 
          tskin    (i,j),   ics      (i,j), &
!
! surface rain rate in [mm/hr] (not in use) and surface vertical wind 
! in [m/s] (not in use)
!
          rainrate_sfc(i,j), wind_lev
!
!             ### READ IN VERTICAL PROFILES ###
! Parameters with the suffix "_lev" are defined at layer interfaces, while
! parameters with the suffix "_avg" are at layer centers. The k-th layer is
! bound between the k-th and (k-1)-th interfaces.
!
          do k = 1, nlyr
            irec = irec + 1
            Read(14,rec=irec) &
!
! height in [km], pressure in [hPa], and air temperature in [K]
! 
             hght_lev(i,j,k), pres_lev(i,j,k), temp_lev(i,j,k), &
!
! relative humidity in [%] and cloud water content in [g/m3]
!                           (= cloud mixing ratio * air density)
!
             rlhm_avg(i,j,k), Wcldw_avg(i,j,k), &
!
! rain water content in [g/m3] and cloud ice content in [g/m3]
!
            Wrain_avg(i,j,k), Wcldi_avg(i,j,k), &
!
! snow ice content in [g/m3] and graupel ice content in [g/m3]
!
            Wsnow_avg(i,j,k), Wgrpl_avg(i,j,k), &
!
! hail ice content in [g/m3] and vertical wind in [m/s] (not in use)
! 
            Whail_avg(i,j,k), wind_lev
!
          end do
       end do
    end do
!
! Define near-surface wind (applicable when background="water") and 
! soil moisture (when background="land").
!
    wind_sfc(:,:) = 6. ; soilH2O(:,:) = 0.
!
! Define visible/IR ground albedo (applicable when background="land")
!
    albd_visir = 0.1
!
! Hydrometeor number concenterations: used only with two-moment
! microphysical schemes. 
!
    Ncldw_avg = 0. ; Ncldi_avg = 0. ; Nrain_avg = 0. 
    Nsnow_avg = 0. ; Ngrpl_avg = 0. ; Nhail_avg = 0.
!
    Return
  End Subroutine rd_CRM
